/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.wmsclient;

import org.apache.log4j.Logger;

import org.exolab.castor.xml.ValidationException;

import java.awt.geom.Rectangle2D;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import java.net.URL;


/**
 * Cliente abstracto que realiza tareas independientes de la versin y delega
 * en un objeto dependiente de la versin las tareas que no puede realizar
 *
 * @author Fernando Gonzlez Corts
 */
public class AbstractWMSClient implements WMSClient {
    private static Logger logger = Logger.getLogger(AbstractWMSClient.class.getName());
    private WMSVersionClient client;
    private Capabilities cachedCapabilities;

    /**
     * Crea un nuevo AbstractWMSClient.
     *
     * @param c Cliente dependiente de la versin
     * @param capabilities Capabilities obtenidas con el cliente que se pasa
     *        como parmetro, con la finalidad de no volver a calcularlas
     */
    public AbstractWMSClient(WMSVersionClient c, Object capabilities) {
        client = c;
        
        cachedCapabilities =  new Capabilities();
        cachedCapabilities.setRoot(capabilities);
        cachedCapabilities.setVersion(c.getVersion());
    }

    /**
     * @see com.iver.cit.gvsig.fmap.wms.WMSClient#getCapabilities(java.net.URL)
     */
    public Capabilities getCapabilities(URL host)
        throws UnsupportedVersionException, IllegalStateException, IOException {
        if (cachedCapabilities != null) {
            return cachedCapabilities;
        } else {
            return client.getCapabilities(host);
        }
    }

    /**
     * @see com.iver.cit.gvsig.fmap.wms.WMSClient#doMapQuery(com.iver.cit.gvsig.fmap.wms.MapQuery)
     */
    public byte[] doMapQuery(MapQuery queryInfo)
        throws WMSException, IOException, NoSuchFieldException, 
            ValidationException {
        InputStream in = null;
        byte[] bytes = null;
        String[] mapURLs = client.getMapURLs();

        if (mapURLs == null) {
            throw new RuntimeException("getCapabilities must be invoked before");
        }

        for (int url = 0; url < mapURLs.length; url++) {
            try {
                URL query = new URL(mapURLs[url] + queryInfo.getQuery());
                logger.debug("Haciendo la peticin: " + query);
                in = query.openStream();

                logger.debug("Obteniendo la respuesta");

                ByteArrayOutputStream outbytes = new ByteArrayOutputStream();
                WMSUtilities.serializar(in, outbytes);
                bytes = outbytes.toByteArray();

                //Se comprueba si la respuesta corresponde a una excepcion del servidor
                if (new String(bytes, 0, bytes.length).toUpperCase().indexOf("EXCEPTION") != -1) {
                    String aux = new String(bytes);
                    logger.debug("Obtenida la siguiente respuesta:\n" + aux);

                    String error = client.parseException(bytes);

                    if (error == null) {
                        return bytes;
                    } else {
                        throw new WMSException(error);
                    }
                } else {
                    return bytes;
                }
            } catch (IOException e) {
                if (url == (mapURLs.length - 1)) {
                    throw e;
                }
            } finally {
                if (in != null) {
                    in.close();
                }
            }
        }

        throw new RuntimeException("No debi llegar nunca a esta linea");
    }

    /**
     * @see com.iver.cit.gvsig.fmap.wms.WMSClient#getVersion()
     */
    public String getVersion() {
        return client.getVersion();
    }

    /**
     * @see com.iver.cit.gvsig.fmap.wms.WMSClient#getBoundingBox(java.lang.String[],
     *      java.lang.String)
     */
    public Rectangle2D getBoundingBox(String[] t, String srs) {
        return client.getBoundingBox(t, srs);
    }

    /**
     * @see com.iver.cit.gvsig.fmap.wms.WMSClient#doFeatureInfo(com.iver.cit.gvsig.fmap.wms.FeatureInfoQuery)
     */
    public byte[] doFeatureInfo(FeatureInfoQuery query)
        throws WMSException, IOException, NoSuchFieldException, 
            ValidationException {
        InputStream in = null;
        byte[] bytes = null;
        String[] infoURLs = client.getInfoURLs();

        if (infoURLs == null) {
            throw new RuntimeException("getCapabilities must be invoked before");
        }

        for (int url = 0; url < infoURLs.length; url++) {
            try {
                URL queryURL = new URL(infoURLs[url] + query.getQuery());
                logger.debug("Realizando la peticion: " + queryURL);
                in = queryURL.openStream();

                ByteArrayOutputStream outbytes = new ByteArrayOutputStream();
                WMSUtilities.serializar(in, outbytes);

                bytes = outbytes.toByteArray();

                String aux = new String(bytes);
                logger.debug("Obtenida la siguiente respuesta: " + aux);

                //Se comprueba si la respuesta se corresponde con una excepcion del servidor
                if (new String(bytes, 0, bytes.length).toUpperCase().indexOf("EXCEPTION") != -1) {
                    String error = client.parseException(bytes);

                    if (error == null) {
                        return bytes;
                    } else {
                        throw new WMSException(error);
                    }
                } else {
                    return bytes;
                }
            } catch (IOException e) {
                if (url == (infoURLs.length - 1)) {
                    throw e;
                }
            } finally {
                in.close();
            }
        }

        throw new RuntimeException("No debi llegar nunca a esta linea");
    }

    /**
     * @see com.iver.wmsclient.WMSClient#getInfoFormats()
     */
    public String[] getInfoFormats() {
        return client.getInfoFormats();
    }

	public MapQuery createQuery() {
		return new MapQuery(client.getQueryHeader());
	}
}

