/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.wmsclient;

import org.exolab.castor.xml.ValidationException;

import java.awt.geom.Rectangle2D;

import java.io.IOException;

import java.net.URL;


/**
 * Interfaz con los mtodos que necesitamos de un cliente WMS. Se pueden aadir
 * ms mtodos en funcin de las necesidades que vayan surgiendo
 *
 * @author Fernando Gonzlez Corts
 */
public interface WMSClient {
    /**
     * Realiza la negociacin de la versin con el servidor WMS, se guarda toda
     * la informacin obtenida de la peticin GetCapabilities si la
     * negociacin es exitosa y lanza una UnsupportedVersionException en caso
     * contrario.
     *
     * @param host DOCUMENT ME!
     *
     * @return Objeto Capabilities. El objeto devuelto tendr root a null si no
     *         se pudo leer la respuesta completamente. Tendr root a null y
     *         version a null si no se pudo leer la version de la respuesta
     *
     * @throws UnsupportedVersionException Si el servidor no soporta ninguna de
     *         las versiones que soporta el cliente
     * @throws IllegalStateException Si se intenta invocar el mtodo y no ha
     *         sido establecido previamente  el parmetros host
     * @throws IOException si se produce un error en la comunicacin con el
     *         servidor
     */
    public Capabilities getCapabilities(URL host)
        throws UnsupportedVersionException, IllegalStateException, IOException;

    /**
     * Realiza una peticion GetMap y devuelve los bytes del fichero retornado
     * por el servidor
     *
     * @param queryInfo Objeto que contiene la informacin necesaria para
     *        realizar la peticion. El campo version de este objeto no viene
     *        definido y ha de ser establecido por las distintas
     *        implementaciones antes de invocar a cualquier otro mtodo del
     *        objeto
     *
     * @return array de bytes del fichero retornado. Si el servidor responde
     *         con una excepcion WMS que el cliente entiende, se lanza una
     *         WMSException con el mensaje de error contenido. Si se lanza una
     *         excepcin que no se entiende se devuelve tal cual. Por tanto es
     *         posible que si no salta la excepcin se devuelva cdigo que no
     *         es una respuesta correcta de informacin correcto
     *
     * @throws WMSException Si el servidor WMS devuelve un error propio del
     *         protocolo
     * @throws IOException Si se produce un error de comunicacin con el
     *         servidor
     * @throws NoSuchFieldException Si no se ha establecido uno de los campos
     *         obligatorios de las peticiones GetMap
     * @throws ValidationException Si no se entiende la respuesta del servidor
     */
    public byte[] doMapQuery(MapQuery queryInfo)
        throws WMSException, IOException, NoSuchFieldException, 
            ValidationException;

    /**
     * Crea un MapQuery con los datos de la versin correspondientes
     * a la versin del cliente. Deber de ser esta mapQuery la
     * que se pase como parmetro a doMapQuery
     *
     * @return DOCUMENT ME!
     */
    public MapQuery createQuery();

    /**
     * Obtiene la version como un string x.y.z
     *
     * @return Nmero de la versin
     */
    public String getVersion();

    /**
     * Obtiene el bounding box de una capa para el SRS  solicitado
     *
     * @param t Array con los names de las capas desde la raiz hasta aquella de
     *        la cual se quiere obtener su bounding box
     * @param srs SRS para la que se quiere el bounding box
     *
     * @return Bounding box de la capa, o null si no se puede encontrar dicho
     *         boundingbox
     */
    public Rectangle2D getBoundingBox(String[] t, String srs);

    /**
     * Realiza una peticin feature info y devuelve un array de bytes con la
     * respuesta
     *
     * @param query Objeto query con la informacin de la peticion. El objeto
     *        MapQuery que hay dentro de este objeto debe ser el mismo (o
     *        contener la misma informacin) que  el que se us para obtener
     *        el mapa sobre el cual se est haciendo la query
     *
     * @return bytes devueltos del servidor
     *
     * @throws WMSException Si el servidor WMS devuelve un error propio del
     *         protocolo
     * @throws IOException Si se produce un error de comunicacin con el
     *         servidor
     * @throws NoSuchFieldException Si no se ha establecido uno de los campos
     *         obligatorios de las peticiones GetFeatureInfo
     * @throws ValidationException Si no se entiende la respuesta del servidor
     */
    public byte[] doFeatureInfo(FeatureInfoQuery query)
        throws WMSException, IOException, NoSuchFieldException, 
            ValidationException;

    /**
     * Obtiene los formatos en los que se le puede pedir  informacin al
     * servidor
     *
     * @return array de strings con los nombres de los formatos que se pueden
     *         pedir al servidor
     */
    public String[] getInfoFormats();
}
