/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.wmsclient;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;

import com.iver.wmsclient.wms_1_0_0.WMS_1_0_0;
import com.iver.wmsclient.wms_1_1_0.WMS_1_1_0;
import com.iver.wmsclient.wms_1_1_1.WMS_1_1_1;


/**
 * Factora abstracta de Clientes WMS
 *
 * @author Fernando Gonzlez Corts
 */
public class WMSClientFactory {
    private static ArrayList versions = new ArrayList();

    static {
        /*
         * Se meten en el array versions las distintas versiones
         * del protocolo en orden descendente
         */
//        versions.add(new WMS_1_3_0());
        versions.add(WMS_1_1_1.class);
        versions.add(WMS_1_1_0.class);
        versions.add(WMS_1_0_0.class);
    }

    /**
     * Mtodo que dada una respuesta de getCapabilities y un iterador sobre una
     * coleccion de WMSClient's ordenada descendentemente devuelve el cliente
     * cuya version es igual o inmediatamente inferior
     *
     * @param caps Capabilities con la respuesta del servidor
     * @param clients Iterador de conjunto ordenado descendientemente
     *
     * @return cliente cuya version es igual o inmediatamente inferior
     * @throws IllegalAccessException
     * @throws InstantiationException
     */
    private static WMSVersionClient getClient(Capabilities caps,
        Iterator clients) throws InstantiationException, IllegalAccessException {
        while (clients.hasNext()) {
            WMSVersionClient client = (WMSVersionClient) ((Class)clients.next()).newInstance();
            int ret = caps.getVersion().compareTo(client.getVersion());

            if (ret >= 0) {
                return client;
            }
        }

        return null;
    }

    /**
     * Establece la versin con la que se comunicar con el servidor y devuelve
     * el objeto Capabilities obtenido con dicha versin
     *
     * @param host maquina con la que se negocia
     *
     * @return instancia de un cliente capaz de negociar con el host que se
     *         pasa como parmetro
     *
     * @throws UnsupportedVersionException Si no se encuentra un cliente
     *         apropiado para el host dado
     * @throws IOException Si se produce un error de entrada salida en la
     *         comunicacin con el servidor
     * @throws IllegalAccessException
     * @throws InstantiationException
     */
    private static WMSClient negotiate(URL host)
        throws UnsupportedVersionException, IOException, InstantiationException, IllegalAccessException {
        if (versions.size() == 0) {
            throw new UnsupportedVersionException("No registered clients");
        }

        Iterator iVersion = versions.iterator();

        if (iVersion.hasNext()) { //Si hay alguna version

            do {
                //Se itera hasta conseguir leer la versin de una respuesta
                Capabilities aux = null;
                WMSVersionClient iV;

                do {
                    if (!iVersion.hasNext()) {
                        throw new UnsupportedVersionException(
                            "Cannot understand server version");
                    }

                    iV = (WMSVersionClient) ((Class)iVersion.next()).newInstance();
                    aux = iV.getCapabilities(host); //Se pide version al servidor
                } while (aux.getVersion() == null); //Se itera hasta que se lee una versin

                int res = aux.getVersion().compareTo(iV.getVersion());

                if (res == 0) { //Si es la misma que nuestra version

                    return new AbstractWMSClient(iV, aux.getRoot());
                } else if (res > 0) { //Si es mayor que nuestra version
                    throw new UnsupportedVersionException(
                        "Server Version too high: " + aux.getVersion());
                } else { //Si es menor que nuestra version
                         //Obtenemos la primera version menor o igual que tengamos

                    WMSVersionClient lower = getClient(aux, iVersion);

                    if (lower == null) { //Si no hay ninguna
                        throw new UnsupportedVersionException(
                            "Lowest server version is " + aux.getVersion());
                    } else {
                        if (lower.getVersion().equals(aux.getVersion())) { //Si hay una igual

                            return new AbstractWMSClient(lower, null);
                        } else { //Si hay una version menor que la que retorno el servidor
                            iV = lower;
                        }
                    }
                }
            } while (iVersion.hasNext());
        }

        throw new UnsupportedVersionException("Negotiation failed");
    }

    /**
     * Establece la versin con la que se comunicar con el servidor y devuelve
     * el objeto Capabilities obtenido con dicha versin
     *
     * @param host maquina con la que se negocia
     *
     * @return instancia de un cliente capaz de negociar con el host que se
     *         pasa como parmetro
     *
     * @throws UnsupportedVersionException Si no se encuentra un cliente
     *         apropiado para el host dado
     * @throws IOException Si se produce un error de entrada salida en la
     *         comunicacin con el servidor
     */
    public static WMSClient getClient(URL host)
        throws UnsupportedVersionException, IOException {
        try {
			return negotiate(host);
		} catch (UnsupportedVersionException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		} catch (InstantiationException e) {
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		}
    }
}
